/* registry.cpp -- realization of Onion Registry library

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */


//#include <iostream>

#include "registry.h"
#include <string>
#include <exception>

using namespace Onion;

Catalog::Catalog() throw(std::bad_alloc)
{
}

Catalog::~Catalog()
{
	std::map<std::string, Catalog *>::iterator i = catalogs.begin();
	while(i != catalogs.end())
	{
		delete i->second;
		catalogs.erase(i);
		i = catalogs.begin();
	}
	
	std::map<std::string, Key *>::iterator iter = keys.begin();
	while(iter != keys.end())
	{
		delete iter->second;
		keys.erase(iter);
		iter = keys.begin();
	}
}

Catalog *Catalog::createCatalog(const std::string &name)
	throw(std::bad_alloc, CantCreateCatalog)
{
	std::string child = name;
	std::string::size_type end = child.size();
	
	try
	{
		if( child.at(end - 1) == '/' )
			child.erase(--end);
		if( child.at(0) == '/' )
			child.erase(0, 1);
	}
	catch(...)
	{
		throw CantCreateCatalog( child );
	}
	
	end = child.find( '/', 0 );
	if( end != std::string::npos )
		throw CantCreateCatalog( child );
	
	std::map<std::string, Catalog *>::iterator iter 
		= catalogs.find( child );
		
	if( iter == catalogs.end() )
		return catalogs[ child ] = new Catalog();

	return iter->second;
}

Catalog *Catalog::getCatalog(const std::string &name)
	throw(CatalogNotFound)
{
	std::string child = name;
	std::string::size_type end = child.size();

	if(child[end - 1] == '/')
		child.erase(--end);
		
	if(child[0] == '/')
		child = child.substr(1, end - 1);
	
	if( child == "" )
		return this;
	
	std::string::size_type pos = child.find('/');
	
	// if not my subcatalog try to find and ask catalog owner
	if(pos != std::string::npos)
	{
		std::string parent = child.substr(0, pos);
		child = child.substr(pos + 1, child.size() - pos);
			
		std::map< std::string, Catalog * >::iterator iter
			= catalogs.find( parent );
		if( iter == catalogs.end() )
			throw CatalogNotFound( parent );

		return iter->second->getCatalog( child );
	}
	
	std::map< std::string, Catalog * >::iterator iter
		= catalogs.find( child );
		
	if( iter == catalogs.end() )
		throw CatalogNotFound( child );
	return iter->second;
}


void Catalog::removeCatalog(const std::string &name)
	throw(CatalogNotFound, CantRemoveCatalog)
{
	Onion::Catalog *catalog = getCatalog( name );
	if( catalog == this )
		throw CantRemoveCatalog( name );
	
	std::string::size_type begin = 0;
	std::string::size_type end = name.size();
	try
	{
	
		if(name.at(0) == '/')
			begin = 1;
		if(name.at(end - 1) == '/')
			end--;
	}
	catch( ... )
	{
		throw CantRemoveCatalog( name );
	}
	
	std::map<std::string, Catalog *>::iterator iter
		= catalogs.find(name.substr(begin, end - begin));
	if(iter == catalogs.end())
		throw CatalogNotFound(name.substr(begin,end - begin));

	delete iter->second;
	catalogs.erase(iter);
}

std::list<std::string> Catalog::getCatalogList() const throw(std::bad_alloc)
{
	std::list<std::string> lst;
	std::map<std::string, Catalog *>::const_iterator iter
		= catalogs.begin();

	while(iter != catalogs.end())
	{
		lst.push_back(iter->first);
		iter++;
	}
	return lst;
}

Key *Catalog::createKey(const std::string &name, const std::string &value)
	throw(std::bad_alloc)
{
	std::map<std::string, Key *>::iterator iter = keys.find(name);
	if(iter != keys.end())
		return iter->second;
	return keys[name] = new Key(value);
}

Key *Catalog::getKey(const std::string &name) const throw(KeyNotFound)
{
	std::map<std::string, Key *>::const_iterator iter = keys.find(name);
	if(iter == keys.end())
		throw KeyNotFound(name);
	return iter->second;
}

void Catalog::removeKey(const std::string &name)
	throw(KeyNotFound, CantRemoveKey)
{
	std::map<std::string, Key *>::iterator iter = keys.find(name);
	if(iter == keys.end())
		throw KeyNotFound(name);

	delete iter->second;
	keys.erase(iter);
}

std::list<std::string> Catalog::getKeyList() const throw(std::bad_alloc)
{
	std::list<std::string> lst;
	std::map<std::string, Key *>::const_iterator iter = keys.begin();
	while(iter != keys.end())
	{
		lst.push_back(iter->first);
		iter++;
	}
	return lst;
}

void Catalog::save( bool recurse ) const
{
	Onion::Storage *storage = Storage::makeStorage( *this );
	storage->open( false );
	save( *storage, recurse );
	storage->close();
}

void Catalog::load( bool recurse )
{
	Onion::Storage *storage = Storage::makeStorage( *this );
	storage->open();
	load( *storage, recurse );
	storage->close();
}


void Catalog::save(Storage &storage, bool recurse ) const
{
	storage.saveMarker( "begin", "catalog" );
	Node::save( storage , recurse );
	std::map< std::string, Key * >::const_iterator iter = keys.begin();
	
	while( iter != keys.end() )
	{
		storage.save( "key", iter->first );
		iter->second->save( storage, recurse );
		++iter;
	}
	
	std::map< std::string, Catalog * >::const_iterator iter2 = catalogs.begin();
	while( iter2 != catalogs.end() )
	{
		storage.save( "catalog", iter2->first );
		iter2->second->save( storage, recurse );
		++iter2;
	}
	storage.saveMarker( "end", "catalog" );	
}

void Catalog::load(Storage &storage, bool recurse )
{
	std::string var, val;
	if( ( storage.load( var, val ) != false )
		|| ( var != "begin" ) || ( val != "catalog" ) )
		throw MarkerExpected( "begin::catalog" );

	Node::load( storage, recurse );

	std::map< std::string, Key * >::iterator key;
	std::map< std::string, Catalog *>::iterator catalog;
	Key *k;
	Catalog *c;
	

	while( storage.load( var, val ) == true )
	{
		if( var == "key" )
		{

			key = keys.find( var );
			if( key != keys.end() )
				k = key->second;
			else
				k = keys[var] = new Key();

			k->load( storage, recurse );
		}
		else if( var == "catalog" )
		{
			catalog = catalogs.find( val );
			if( catalog != catalogs.end() )
				c = catalog->second;
			else
				c = catalogs[val] = new Catalog();
			c->load( storage, recurse );
		}
	}
	if( ( var != "end" ) || ( val != "catalog" ) )
		throw MarkerExpected( "end::catalog" );
}

// <EOF>
