/* registry.cpp -- realization of Onion Registry library

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */


//#include <iostream>
#include <memory>

#include "registry.h"

using namespace Onion;

Node::Node() throw(std::bad_alloc)
{
}

Node::~Node()
{
	std::map<std::string, MetaKey *>::iterator iter = metakeys.begin();
	
	while(iter != metakeys.end())
	{
		delete iter->second;
		iter++;
	}
}

MetaKey *Node::createMetaKey(const std::string &name, const std::string &value)
	throw(std::bad_alloc, MetaKeyExist)
{
	std::map<std::string, MetaKey *>::iterator iter = metakeys.find(name);
	if(iter != metakeys.end())
		throw MetaKeyExist(name);
	return metakeys[name] = new MetaKey(value);
}

MetaKey *Node::getMetaKey(const std::string &name) const throw(MetaKeyNotFound)
{
	std::map<std::string, MetaKey *>::const_iterator iter
		= metakeys.find(name);
	if(iter == metakeys.end())
		throw MetaKeyNotFound(name);
	return iter->second;
}

void Node::removeMetaKey(const std::string &name) throw(MetaKeyNotFound)
{
	std::map<std::string, MetaKey *>::iterator iter = metakeys.find(name);
	if(iter == metakeys.end())
		throw MetaKeyNotFound(name);
	
	metakeys.erase(iter);
}

std::list<std::string> Node::getMetaKeyList() const throw(std::bad_alloc)
{
	std::list<std::string> lst;
	std::map<std::string, MetaKey *>::const_iterator iter = metakeys.begin();

	while(iter != metakeys.end())
	{
		lst.push_back(iter->first);
		iter++;
	}
	return lst;
}


void Node::save( bool recurse ) const
{
	throw OperationNotSupported( "Node::save() const" );
}

void Node::load( bool recurse )
{
	throw OperationNotSupported( "Node::load()" );
}


void Node::save( Storage &storage, bool recurse ) const
{
	storage.saveMarker( "begin", "node" );
	
	std::map< std::string, MetaKey * >::const_iterator iter
		= metakeys.begin();

	while( iter != metakeys.end() )
	{
		if(iter->second->getOwnerParent() == false)
			storage.save( iter->first, iter->second->getValue() );

		++iter;
	}
	
	storage.saveMarker( "end", "node" );
}

void Node::load(Storage &storage, bool recurse )
{
	std::string var, val;
	if( ( storage.load( var, val ) != false )
		|| ( var != "begin" ) || ( val != "node" ) )
		throw MarkerExpected( "begin::node" );
		
	while( storage.load( var, val ) == true )
	{
		std::map< std::string, MetaKey *>::iterator iter
			= metakeys.find( var );

		if(iter != metakeys.end() )
			*iter->second = val;
		else
			metakeys[var] = new MetaKey( val );
	}

	if( ( var != "end" ) || (val != "node" ) )
		throw MarkerExpected( "end::node" );
}

// <EOF>
