/* regitry.h -- interface to Onion Registry library

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

///////////////////////////////////////////////////////////////////////////////
/// @file registry.h
/// @brief library header file
///////////////////////////////////////////////////////////////////////////////

#ifndef REGISTRY_H
#define REGISTRY_H

#include <string>
#include <list>
#include <map>

///////////////////////////////////////////////////////////////////////////////
/// @brief Onion project namespace
///////////////////////////////////////////////////////////////////////////////
namespace Onion
{
	class Node;
	class Catalog;
	class Key;
	class MetaKey;
	class Storage;

///////////////////////////////////////////////////////////////////////////////
/// @defgroup libExceptions Exceptions
/// @{
///
/// This exceptions throws by Onion::registry library routines
///////////////////////////////////////////////////////////////////////////////
	
///////////////////////////////////////////////////////////////////////////////
/// @brief base library exception
///////////////////////////////////////////////////////////////////////////////
	class RegistryException {};
	
	class ExceptionWithValue : public RegistryException
	{
	public:
		ExceptionWithValue(const std::string &value) 
			throw(std::bad_alloc) : val(value) {};
		inline std::string getValue() {return val;};
		inline operator std::string() const
			throw(std::bad_alloc) {return val;};
	private:
		std::string val;
	}; // class ExceptionWithString

///////////////////////////////////////////////////////////////////////////////
/// @brief throws if called operation not supported
///////////////////////////////////////////////////////////////////////////////
	//class OperationNotSupported : public RegistryException {}
	class OperationNotSupported : public ExceptionWithValue
	{
	public:
		OperationNotSupported(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // OperationNotSupported

///////////////////////////////////////////////////////////////////////////////
/// @brief throws if Node defined by MetaKey "Storage" not supported by library
///////////////////////////////////////////////////////////////////////////////
	class StorageNotSupported : public ExceptionWithValue
	{
	public:
		StorageNotSupported(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // StorageNotSupported

///////////////////////////////////////////////////////////////////////////////
/// @brief throws if requed Catalog not exist
///////////////////////////////////////////////////////////////////////////////
	class CatalogNotFound : public ExceptionWithValue
	{
	public:
		CatalogNotFound(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // CatalogNotFound

///////////////////////////////////////////////////////////////////////////////
/// @brief throws if Key not found.
///////////////////////////////////////////////////////////////////////////////
	class KeyNotFound : public ExceptionWithValue
	{
	public:
		KeyNotFound(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // KeyNotFound

///////////////////////////////////////////////////////////////////////////////
/// @brief throws if MetaKey not found.
///////////////////////////////////////////////////////////////////////////////
	class MetaKeyNotFound : public ExceptionWithValue
	{
	public:
		MetaKeyNotFound(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // MetaKeyNotFound

///////////////////////////////////////////////////////////////////////////////
/// @brief throws when trying to create existed MetaKey
///////////////////////////////////////////////////////////////////////////////
	class MetaKeyExist : public ExceptionWithValue
	{
	public:
		MetaKeyExist(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // MetaKeyExist

///////////////////////////////////////////////////////////////////////////////
/// @brief throws when trying to save unknown marker
///
/// Known markers:
/// beginCatalog, endCatalog, beginNode, endNode
///////////////////////////////////////////////////////////////////////////////
	class UnknownMarker : public ExceptionWithValue
	{
	public:
		UnknownMarker(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // UnknownMarker

///////////////////////////////////////////////////////////////////////////////
/// @brief throws when expecing load marker, but loaded variable
///////////////////////////////////////////////////////////////////////////////
	class MarkerExpected : public ExceptionWithValue
	{
	public:
		MarkerExpected(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // MarkerExpected

///////////////////////////////////////////////////////////////////////////////
/// @brief throws when expecing load variable, but loaded marker
///////////////////////////////////////////////////////////////////////////////
	class VariableExpected : public ExceptionWithValue
	{
	public:
		VariableExpected(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // VariableExpected

///////////////////////////////////////////////////////////////////////////////
/// @brief throws when expecing load certain variable and loads something else
///////////////////////////////////////////////////////////////////////////////
	class BadVariable : public ExceptionWithValue
	{
	public:
		BadVariable(const std::string &value)
			throw(std::bad_alloc) : ExceptionWithValue(value) {};
	}; // BadVariable

///////////////////////////////////////////////////////////////////////////////
/// @brief throws when meet with unknown error
///////////////////////////////////////////////////////////////////////////////

	class ErrorException : public ExceptionWithValue
	{
	public:
		ErrorException(const std::string &value)
			: ExceptionWithValue(value) {};
	}; // class ErrorException

///////////////////////////////////////////////////////////////////////////////
/// @brief throwns then lib cant create catalog
///////////////////////////////////////////////////////////////////////////////
	class  CantCreateCatalog : public ExceptionWithValue
	{
	public:
		CantCreateCatalog( const std::string &value )
			: ExceptionWithValue( value ) {};
	}; // class CantCreateCatalog

///////////////////////////////////////////////////////////////////////////////
/// @brief throwns then try removed key is absense
///////////////////////////////////////////////////////////////////////////////
	class  CantRemoveKey : public ExceptionWithValue
	{
	public:
		CantRemoveKey( const std::string &value )
			: ExceptionWithValue( value ) {};
	}; // class CantCreateKey
	
///////////////////////////////////////////////////////////////////////////////
/// @brief throwns then try removed catalog is absense
///////////////////////////////////////////////////////////////////////////////
	class  CantRemoveCatalog : public ExceptionWithValue
	{
	public:
		CantRemoveCatalog( const std::string &value )
			: ExceptionWithValue( value ) {};
	}; // class CantCreateCatalog

///////////////////////////////////////////////////////////////////////////////
/// @brief throwns then storage cant create object to writing of registry
///////////////////////////////////////////////////////////////////////////////
	class  CantCreateStorage : public ExceptionWithValue
	{
	public:
		CantCreateStorage( const std::string &value )
			: ExceptionWithValue( value ) {};
	}; // class CantCreateStorage

///////////////////////////////////////////////////////////////////////////////
/// @brief throwns then storage cant being open
///////////////////////////////////////////////////////////////////////////////
	class  CantOpenStorage : public ExceptionWithValue
	{
	public:
		CantOpenStorage( const std::string &value )
			: ExceptionWithValue( value ) {};
	}; // class CantOpenStorage

///////////////////////////////////////////////////////////////////////////////
/// @brief throwns then storage already open when tryin open again
///////////////////////////////////////////////////////////////////////////////
	class  StorageAlreadyOpen : public ExceptionWithValue
	{
	public:
		StorageAlreadyOpen( const std::string &value )
			: ExceptionWithValue( value ) {};
	}; // class StorageAlreadyOpen
	
///////////////////////////////////////////////////////////////////////////////
/// @} end of exception group
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/// @defgroup libClasses Library classes
/// @{
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
/// @brief create Registry object for interaction with registry
///////////////////////////////////////////////////////////////////////////////
	class Registry{
	public:
///////////////////////////////////////////////////////////////////////////////
/// @brief default constructor
///////////////////////////////////////////////////////////////////////////////
		Registry() throw (std::bad_alloc);
		
///////////////////////////////////////////////////////////////////////////////
/// @brief destructor
///////////////////////////////////////////////////////////////////////////////
		~Registry();

///////////////////////////////////////////////////////////////////////////////
/// @brief returns root of registry
///////////////////////////////////////////////////////////////////////////////
		Catalog *getRoot() throw();

	private:

///////////////////////////////////////////////////////////////////////////////
/// @brief prevent copying
/// @{
///////////////////////////////////////////////////////////////////////////////
		Registry(const Registry&);
		Registry &operator=(const Registry &);
///////////////////////////////////////////////////////////////////////////////
/// @}
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/// @brief current root Catalog
///////////////////////////////////////////////////////////////////////////////

		Catalog *root;
	}; // class Registry
	
///////////////////////////////////////////////////////////////////////////////
/// @brief base class for Catalog and Key
///////////////////////////////////////////////////////////////////////////////
	class Node
	{
	public:
		Node() throw(std::bad_alloc);
///////////////////////////////////////////////////////////////////////////////
/// @brief default destructor
///////////////////////////////////////////////////////////////////////////////
		virtual ~Node();
		
///////////////////////////////////////////////////////////////////////////////
/// @brief create new one MetaKey and return pointer to them.
///////////////////////////////////////////////////////////////////////////////
		MetaKey *createMetaKey(const std::string &name,
			const std::string &value)
			throw(std::bad_alloc, MetaKeyExist);


///////////////////////////////////////////////////////////////////////////////
/// @brief returns pointer to MetaKey if exist.
///////////////////////////////////////////////////////////////////////////////
		MetaKey *getMetaKey(const std::string &name) const
			throw(MetaKeyNotFound);


///////////////////////////////////////////////////////////////////////////////
/// @brief removes MetaKey 
///////////////////////////////////////////////////////////////////////////////
		void removeMetaKey(const std::string &name)
			throw(MetaKeyNotFound);


///////////////////////////////////////////////////////////////////////////////
/// @brief returns list of existed MetaKeys
///////////////////////////////////////////////////////////////////////////////
		std::list<std::string> getMetaKeyList() const 
			throw(std::bad_alloc);

///////////////////////////////////////////////////////////////////////////////
/// @brief save node
///////////////////////////////////////////////////////////////////////////////
		virtual void save( bool recurse ) const;

///////////////////////////////////////////////////////////////////////////////
/// @brief load node
///////////////////////////////////////////////////////////////////////////////
		virtual void load( bool recurse );
		
///////////////////////////////////////////////////////////////////////////////
/// @brief stores MetaKey to Storage
///////////////////////////////////////////////////////////////////////////////
		virtual void save(Storage &storage, bool recurse ) const;
		
///////////////////////////////////////////////////////////////////////////////
/// @brief loads all MetaKeys from Storage
///////////////////////////////////////////////////////////////////////////////
		virtual void load(Storage &storage, bool recurse );
		
	protected:
///////////////////////////////////////////////////////////////////////////////
/// @brief list of metakeys
///////////////////////////////////////////////////////////////////////////////
		std::map<std::string, MetaKey *> metakeys;
	}; // class Node



///////////////////////////////////////////////////////////////////////////////
/// @brief Catalog may contains Keys and Catalogs
///////////////////////////////////////////////////////////////////////////////
	class Catalog : public Node
	{
	public:
	
///////////////////////////////////////////////////////////////////////////////
/// @brief default constructor		
///////////////////////////////////////////////////////////////////////////////
		Catalog() throw(std::bad_alloc);

///////////////////////////////////////////////////////////////////////////////
/// @brief destructor
///////////////////////////////////////////////////////////////////////////////
		virtual ~Catalog();


///////////////////////////////////////////////////////////////////////////////
/// @brief create new one Catalog
///////////////////////////////////////////////////////////////////////////////
		Catalog *createCatalog(const std::string &name)
			throw(std::bad_alloc, CantCreateCatalog);

///////////////////////////////////////////////////////////////////////////////
/// @brief returns existed Catalog
///////////////////////////////////////////////////////////////////////////////
		Catalog *getCatalog(const std::string &name)
			throw(CatalogNotFound);

///////////////////////////////////////////////////////////////////////////////
/// @brief remove existing catalog
///////////////////////////////////////////////////////////////////////////////
		void removeCatalog(const std::string &name)
			throw(CatalogNotFound, CantRemoveCatalog);

///////////////////////////////////////////////////////////////////////////////
/// @brief returns list of existed directories names
///////////////////////////////////////////////////////////////////////////////
		std::list<std::string> getCatalogList() const
			throw(std::bad_alloc);


///////////////////////////////////////////////////////////////////////////////
/// @brief create new Key
///////////////////////////////////////////////////////////////////////////////
		Key *createKey(const std::string &name,
			const std::string &value) throw(std::bad_alloc);


///////////////////////////////////////////////////////////////////////////////
/// @brief returns existed Key
///////////////////////////////////////////////////////////////////////////////
		Key *getKey(const std::string &name) const
			throw(KeyNotFound);


///////////////////////////////////////////////////////////////////////////////
/// @brief removes existed Key
///////////////////////////////////////////////////////////////////////////////
		void removeKey( const std::string &name )
			throw( KeyNotFound, CantRemoveKey );


///////////////////////////////////////////////////////////////////////////////
/// @brief returns list of names existed Keys
///////////////////////////////////////////////////////////////////////////////
		std::list<std::string> getKeyList() const throw(std::bad_alloc);


///////////////////////////////////////////////////////////////////////////////
/// @brief save catalog
///////////////////////////////////////////////////////////////////////////////
		void save( bool recurse ) const;

///////////////////////////////////////////////////////////////////////////////
/// @brief load catalog
///////////////////////////////////////////////////////////////////////////////
		void load( bool recurse );

///////////////////////////////////////////////////////////////////////////////
/// @brief saves catalog to storage
///////////////////////////////////////////////////////////////////////////////
		void save(Storage &storage, bool recurse ) const;

///////////////////////////////////////////////////////////////////////////////
/// @brief loads catalog from storage
///////////////////////////////////////////////////////////////////////////////
		void load(Storage &storage, bool recurse );

	private:
///////////////////////////////////////////////////////////////////////////////
/// @brief protect from copying of Catalog object
/// @{
///////////////////////////////////////////////////////////////////////////////
		Catalog(const Catalog &);
		Catalog &operator=(const Catalog &);
///////////////////////////////////////////////////////////////////////////////
/// @}
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/// @brief list of existed directories
///////////////////////////////////////////////////////////////////////////////
		std::map<std::string, Catalog *> catalogs;
		
///////////////////////////////////////////////////////////////////////////////
/// @brief list of existed keys
///////////////////////////////////////////////////////////////////////////////
		std::map<std::string, Key *> keys;
	}; // class Catalog
	
///////////////////////////////////////////////////////////////////////////////
/// @brief this store Key value	
///////////////////////////////////////////////////////////////////////////////
	class Key : public Node
	{
	public:
///////////////////////////////////////////////////////////////////////////////
/// @brief default constructor
///////////////////////////////////////////////////////////////////////////////
		Key() throw(std::bad_alloc);
		
///////////////////////////////////////////////////////////////////////////////
/// @brief construct from std::string as value
///////////////////////////////////////////////////////////////////////////////
		Key(const std::string &value) throw(std::bad_alloc);
		
///////////////////////////////////////////////////////////////////////////////
/// @brief assign value
///////////////////////////////////////////////////////////////////////////////
		Key& operator= (const std::string &value) throw();

///////////////////////////////////////////////////////////////////////////////
/// @brief get Key value
///////////////////////////////////////////////////////////////////////////////

		std::string getValue() const throw(std::bad_alloc);

///////////////////////////////////////////////////////////////////////////////
/// @brief use Key as std::string
///////////////////////////////////////////////////////////////////////////////
		operator std::string() const throw(std::bad_alloc);

///////////////////////////////////////////////////////////////////////////////
/// @brief save key
///////////////////////////////////////////////////////////////////////////////
		void save( bool recurse ) const;

///////////////////////////////////////////////////////////////////////////////
/// @brief load key
///////////////////////////////////////////////////////////////////////////////
		void load( bool recurse );


///////////////////////////////////////////////////////////////////////////////
/// @brief stores key value to storage
///////////////////////////////////////////////////////////////////////////////
		void save(Storage &storage, bool recurse ) const;
		
///////////////////////////////////////////////////////////////////////////////
/// @brief loads key from storage
///////////////////////////////////////////////////////////////////////////////
		void load(Storage &storage, bool recurse );
	private:
///////////////////////////////////////////////////////////////////////////////
/// @brief protect from copying of Key objects
/// @{
///////////////////////////////////////////////////////////////////////////////
		Key(const Key &);
		Key &operator=(const Key&);
///////////////////////////////////////////////////////////////////////////////
/// @}
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/// @brief Key value
///////////////////////////////////////////////////////////////////////////////
		std::string value;
	}; // class Key

///////////////////////////////////////////////////////////////////////////////
/// @brief stores additional inforamtion for Node children
///////////////////////////////////////////////////////////////////////////////
	class MetaKey
	{
	public:
///////////////////////////////////////////////////////////////////////////////
/// @brief default constructor
///////////////////////////////////////////////////////////////////////////////
		MetaKey() throw(std::bad_alloc);
		
///////////////////////////////////////////////////////////////////////////////
/// @brief construct from std::string as value
///////////////////////////////////////////////////////////////////////////////
		MetaKey(const std::string &value) throw(std::bad_alloc);

///////////////////////////////////////////////////////////////////////////////
/// @brief assign value
///////////////////////////////////////////////////////////////////////////////
		MetaKey& operator= (const std::string &value) throw();

///////////////////////////////////////////////////////////////////////////////
/// @brief get MetaKey value
///////////////////////////////////////////////////////////////////////////////
		std::string getValue() const throw(std::bad_alloc);

///////////////////////////////////////////////////////////////////////////////
/// @brief use MetaKey as std::string
///////////////////////////////////////////////////////////////////////////////
		operator std::string() const throw(std::bad_alloc);
		
///////////////////////////////////////////////////////////////////////////////
/// @brief 
///////////////////////////////////////////////////////////////////////////////
		bool getOwnerParent() const throw();

///////////////////////////////////////////////////////////////////////////////
/// @brief 
///////////////////////////////////////////////////////////////////////////////
		void setOwnerParent( bool owner_parent) throw();

	private:
///////////////////////////////////////////////////////////////////////////////
/// @brief protect from copying of MetaKey objects
/// @{
///////////////////////////////////////////////////////////////////////////////
		MetaKey(const MetaKey &);
		MetaKey &operator=(const MetaKey &);
///////////////////////////////////////////////////////////////////////////////
/// @}
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/// @brief MetaKey value
///////////////////////////////////////////////////////////////////////////////
		std::string value;
		bool owner_parent;
	}; // class MetaKey
	
///////////////////////////////////////////////////////////////////////////////
/// @brief interface for storage extensions
///////////////////////////////////////////////////////////////////////////////
	class Storage
	{
	public:
///////////////////////////////////////////////////////////////////////////////
/// @brief factory of Storage, you should delete any made storage
///////////////////////////////////////////////////////////////////////////////
		static Storage *makeStorage(const Node &node)
			throw(OperationNotSupported, StorageNotSupported,
				MetaKeyNotFound);
		
///////////////////////////////////////////////////////////////////////////////
/// @brief base class should have virtual destructor
///////////////////////////////////////////////////////////////////////////////
		virtual ~Storage();
		
///////////////////////////////////////////////////////////////////////////////
/// @brief open before reading or writing
///////////////////////////////////////////////////////////////////////////////
		virtual void open(bool read = true) = 0;
		
///////////////////////////////////////////////////////////////////////////////
/// @brief close storage
///////////////////////////////////////////////////////////////////////////////
		virtual void close() = 0;

		///////////////////////////////////////////////////////////////
		/// @brief save pairs list
		///////////////////////////////////////////////////////////////
		virtual void save(const std::string &variable,
			const std::string &value) = 0;
		
		///////////////////////////////////////////////////////////////
		/// @brief save named marker
		///////////////////////////////////////////////////////////////
		virtual void saveMarker(const std::string &marker,
			const std::string &value) = 0;
		

		///////////////////////////////////////////////////////////////
		/// @brief if returns true, variable and value returned,
		/// and if returns false, then marker and value returned
		///////////////////////////////////////////////////////////////
		virtual bool load( std::string &variable,
			std::string &value) = 0;
	};

///////////////////////////////////////////////////////////////////////////////
/// @} end of library classes group
///////////////////////////////////////////////////////////////////////////////
	
}; // namespace Registry

#endif // REGISTRY_H

// <EOF>
