#ifndef TESTS_CATALOGTEST_H
#define TESTS_CATALOGTEST_H

#include "cppunit/extensions/HelperMacros.h"
#include "../src/registry.h"
//#include <iostream>
#include <algorithm>

class CatalogTest : public CppUnit::TestFixture
{
	CPPUNIT_TEST_SUITE( CatalogTest );

	CPPUNIT_TEST( testCreateAndGetCatalog );
	CPPUNIT_TEST_EXCEPTION( testCatalogNotFound, Onion::CatalogNotFound );

	CPPUNIT_TEST_EXCEPTION( testCantRemoveCatalog,	Onion::CantRemoveCatalog );
	CPPUNIT_TEST( testRemoveCatalog );
	
	CPPUNIT_TEST( testCreateAndGetKey);	
	CPPUNIT_TEST_EXCEPTION( testKeyNotFound, Onion::KeyNotFound );
	CPPUNIT_TEST( testRemoveKey );
	
	CPPUNIT_TEST( testGetKeyList );
	
	CPPUNIT_TEST( testSave );
	CPPUNIT_TEST( testLoad );


	
	
	
	CPPUNIT_TEST_SUITE_END();	

protected:
	Onion::Catalog *m_catalog;
	Onion::Registry *m_registry;
public:
	int countTestCases() const
	{
		return 1;
	}
	
	void setUp()
	{
		m_registry = new Onion::Registry();
		Onion::Catalog *root = m_registry->getRoot();
		m_catalog = root->createCatalog( "catalog" );
	}
	
	void tearDown()
	{
		delete m_registry;
	}

	void testCreateAndGetCatalog()
	{
		Onion::Catalog *catalog = m_catalog->createCatalog( "/catalog/" );
		CPPUNIT_ASSERT( catalog != 0 );
		CPPUNIT_ASSERT( catalog == m_catalog->getCatalog( "/catalog/" ) );
		CPPUNIT_ASSERT( catalog == m_catalog->getCatalog( "catalog" ) );
		CPPUNIT_ASSERT( catalog == m_catalog->getCatalog( "catalog/" ) );
		CPPUNIT_ASSERT( catalog == m_catalog->getCatalog( "/catalog" ) );
	}

	void getCatalogList()
	{
		m_catalog->createCatalog( "catalog" );
		std::list< std::string > lst = m_catalog->getCatalogList();
		CPPUNIT_ASSERT( *lst.begin() == "catalog" );
		CPPUNIT_ASSERT( ++lst.begin() == lst.end() );
	}	

	void testCatalogNotFound()
	{
		m_catalog->getCatalog( "not existed" );
	}
	
	void testRemoveCatalog()
	{
		Onion::Catalog *catalog = m_catalog->createCatalog( "catalog" );
		CPPUNIT_ASSERT( catalog != 0 );
		m_catalog->removeCatalog( "catalog" );
		std::list< std::string > lst = m_catalog->getCatalogList();
		CPPUNIT_ASSERT( lst.begin() == lst.end() );
	}

	void testCantRemoveCatalog()
	{
		m_catalog->removeCatalog( "" );
	}
	
	void testCreateAndGetKey()
	{
		Onion::Key *key = m_catalog->createKey( "key", "value" );
		CPPUNIT_ASSERT( key != 0 );
		CPPUNIT_ASSERT( m_catalog->getKey( "key" ) == key );
	}
	
	void testKeyNotFound()
	{
		m_catalog->getKey( "key not exist" );
	}
	
	void testRemoveKey()
	{
		Onion::Key *key = m_catalog->createKey( "key", "value" );
		CPPUNIT_ASSERT( key != 0 );
		CPPUNIT_ASSERT( m_catalog->getKey( "key" ) == key );
		m_catalog->removeKey( "key" );
		std::list<std::string> lst = m_catalog->getKeyList();
		CPPUNIT_ASSERT( lst.begin() == lst.end() );
	}
	
	void testGetKeyList()
	{
		std::list<std::string> lst = m_catalog->getKeyList();
		CPPUNIT_ASSERT( lst.begin() == lst.end() );
		Onion::Key *key = m_catalog->createKey( "key", "value" );
		CPPUNIT_ASSERT( key != 0 );
		CPPUNIT_ASSERT( m_catalog->getKey( "key" ) == key );
		lst = m_catalog->getKeyList();
		CPPUNIT_ASSERT( *lst.begin() == "key" );
		CPPUNIT_ASSERT( ++lst.begin() == lst.end() );
	}
	
	void testSave()
	{
		Onion::MetaKey *mkey;
		mkey = m_catalog->createMetaKey( "storage", "SimpleStorage" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );
		
		mkey = m_catalog->createMetaKey( "filename", "testCatalog.reg" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		mkey = m_catalog->createMetaKey( "metakey", "saved metakey" );
		CPPUNIT_ASSERT( mkey != 0 );
		Onion::Key *key = m_catalog->createKey( "key", "saved key" );
		CPPUNIT_ASSERT( key != 0 );
		Onion::Catalog *catalog = m_catalog->createCatalog( "catalog" );
		CPPUNIT_ASSERT( catalog != 0 );

		m_catalog->save( false );
	}
	
	void testLoad()
	{
		Onion::MetaKey *mkey;
		mkey = m_catalog->createMetaKey( "storage", "SimpleStorage" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );
		
		mkey = m_catalog->createMetaKey( "filename", "testCatalog.reg" );
		CPPUNIT_ASSERT( mkey != 0 );
		mkey->setOwnerParent( true );

		m_catalog->load( false );

		mkey = m_catalog->getMetaKey( "metakey" );
		CPPUNIT_ASSERT( mkey != 0 );
		CPPUNIT_ASSERT( mkey->getValue() == "saved metakey" );
		
		Onion::Key *key = m_catalog->getKey( "key" );
		CPPUNIT_ASSERT( key != 0 );
		CPPUNIT_ASSERT( key->getValue() == "saved key" );
		
		Onion::Catalog *catalog = m_catalog->getCatalog( "catalog" );
		CPPUNIT_ASSERT( catalog != 0 );
	}
	
};

#endif // TESTS_REGISTRY_H

// <EOF>
